/******************************************************************************
 *                                L1b_file.h                                  *
 *                        ---------------------------                         *
 *   start                : September 21th-2008                               *
 *   author               : Lars Stenseng                                     *
 *   email                : stenseng@space.dtu.dk                             *
 *   last update          : October 28th-2008                                 *
 ******************************************************************************/

#ifndef L1B_FILE_H__
#define L1B_FILE_H__

using namespace std;

#include <cmath>

#include <QtGlobal>
#include <QFile>
#include <QDataStream>
#include <QString>
#include <QStringList>
#include <QHash>
#include <QVector>
#include <QDate>

class AsirasData
{public:
  AsirasData();
  ~AsirasData();
  int open(const QString &name);
  qint64 size() {return(privAsirasFile.size());};
  int records() {return(privDataRecords);};
  int echoes() {return(privDataRecords*20);};
  int getL1bRecord();
  int getL1bRecords();
  QDate date(int measurement);
  double resolution();
  QString opMode();
  double lightSpeed() {return(299792458.0);};
  int powerEchoSize();
  double scaleFactor(int i){return(linearScaleFactorA.at(i)*1e-9*pow(2.0, powerScaleFactorB.at(i)));};
  double measurementDerivedElevation(int echo, double measurement);
  double measurementDerivedElevation(int echo, int measurement);
  double minSeconds() {return(privMinSeconds);};
  double maxSeconds() {return(privMaxSeconds);};
  double minCBaselineLatitude() {return(privMinCBaselineLatitude);};
  double maxCBaselineLatitude() {return(privMaxCBaselineLatitude);};
  double minCBaselineLongitude() {return(privMinCBaselineLongitude);};
  double maxCBaselineLongitude() {return(privMaxCBaselineLongitude);};
  double minCBaselineAltitude() {return(privMinCBaselineAltitude);};
  double maxCBaselineAltitude() {return(privMaxCBaselineAltitude);};
  double minDerivedSurfaceElevation() {return(privMinDerivedSurfaceElevation);};
  double maxDerivedSurfaceElevation() {return(privMaxDerivedSurfaceElevation);};

  enum {HAM=1, LAM=2, LAMA=3, LAMA_W=4};

  QHash<QString, QString> mainProductHeader;
  QHash<QString, QString> specificProductHeader;
  QHash<QString, QString> dataSetDescriptor;

  QVector<qint32> days;
  QVector<double> seconds;
  QVector<quint32> instrumentConfig;
  QVector<quint32> burstCounter;
  QVector<double> cBaselineLatitude;
  QVector<double> cBaselineLongitude;
  QVector<double> cBaselineAltitude;
  QVector<double> altitudeRate;
  QVector<double> xVelocityITRF;
  QVector<double> yVelocityITRF;
  QVector<double> zVelocityITRF;
  QVector<double> xRealAntBeamDir;
  QVector<double> yRealAntBeamDir;
  QVector<double> zRealAntBeamDir;
  QVector<double> xInterferometerBaseline;
  QVector<double> yInterferometerBaseline;
  QVector<double> zInterferometerBaseline;
  QVector<quint32> measurementConfident;

  QVector<double> windowDelay;
  QVector<double> oCOGWidth;
  QVector<double> retrackerRange;
  QVector<double> derivedSurfaceElevation;
  QVector<double> aGCChannel1;
  QVector<double> aGCChannel2;
  QVector<double> totalFixedGain1;
  QVector<double> totalFixedGain2;
  QVector<double> transmitPower;
  QVector<double> dopplerRangeCorrection;
  QVector<double> instrumentRangeCorrectionCh1;
  QVector<double> instrumentRangeCorrectionCh2;
  QVector<double> internalPhaseCorrection;
  QVector<double> externalPhaseCorrection;
  QVector<double> noisePower;
  QVector<double> roll;
  QVector<double> pitch;
  QVector<double> yaw;
  QVector<double> heading;
  QVector<double> stdDevRoll;
  QVector<double> stdDevPitch;
  QVector<double> stdDevYaw;

  QVector< QVector<quint16> > powerEcho;
  QVector<qint32> linearScaleFactorA;
  QVector<qint32> powerScaleFactorB;
  QVector<quint16> numberOfMultilookedEchoes;
  QVector<quint16> flags;
  QVector< QVector<quint16> > beamBehaviourParameters;
  QVector< QVector<double> > coherenceEcho;
  QVector< QVector<double> > phaseDifferenceEcho;

 private:
  QFile privAsirasFile;
  QDataStream privAsirasDataStream;
  int privDataRecords;
  int privCurrentRecord;
  QDate privStartEpoch;
  int privOpMode;
  double privMinSeconds, privMaxSeconds;
  double privMinCBaselineLatitude, privMaxCBaselineLatitude;
  double privMinCBaselineLongitude, privMaxCBaselineLongitude;
  double privMinCBaselineAltitude, privMaxCBaselineAltitude;
  double privMinDerivedSurfaceElevation, privMaxDerivedSurfaceElevation;
};

#endif
