/******************************************************************************
 *                          main.h                                            *
 *                        ---------------------------                         *
 *   start                : May 18th-2009                                     *
 *   author               : Lars Stenseng                                     *
 *   email                : stenseng@space.dtu.dk                             *
 *   last update          : May 18th-2009                                     *
 ******************************************************************************/

#include "L1b_file.h"
#include "gravsoftGrid.h"
#include <QFile>
#include <QFileInfo>
#include <QTextStream>
#include <iostream>

using namespace std;

int main(int argc, char *argv[])
{
    QString inputFileName, geoideFileName;
    bool outputAll=false, useGeoide=false;

    if(argc<2)
    {
        cerr << "Usage: Asiras_L1b_info [-g geoide_file] [-a] asiras_l1b_file";
        exit(1);
    }
    else
    {
        for(int i=1; i<argc; i++)
        {if(argv[i][0]=='-' || argv[i][0]=='/')
            {
                if(argv[i][1]=='g' || argv[i][1]=='G')
                {
                    useGeoide=true;
                    i++;
                    geoideFileName=argv[i];
                }
                else if(argv[i][1]=='a' || argv[i][1]=='A')
                {
                    outputAll=true;
                }
                else
                {
                    cerr << "Option: " << argv[i] << " is unknown!";
                    exit(2);
                }
            }
            else
            {
                inputFileName=argv[i];
            }
        }
    }

    QFileInfo inputFileInfo(inputFileName);
    GravsoftGrid geoide;

    if(useGeoide) geoide.open(geoideFileName);

    AsirasData asirasData;
    asirasData.open(inputFileName);
    asirasData.getL1bRecords();

    QFile outputFile;
    QTextStream outputStream;
    
    outputFile.setFileName(inputFileInfo.baseName()+".info");
    outputFile.open(QIODevice::WriteOnly);
    outputStream.setDevice(&outputFile);

    if(outputAll) for(int echo=0; echo<asirasData.echoes(); echo++)
    {
        outputStream <<  fixed
                << qSetFieldWidth(10) << qSetRealNumberPrecision(3)  << asirasData.seconds.at(echo)
                << qSetFieldWidth(12) << qSetRealNumberPrecision(0)  << asirasData.instrumentConfig.at(echo)
                << qSetFieldWidth(6)  << qSetRealNumberPrecision(0)  << asirasData.burstCounter.at(echo)
                << qSetFieldWidth(13) << qSetRealNumberPrecision(7)  << asirasData.cBaselineLatitude.at(echo)
                << qSetFieldWidth(13) << qSetRealNumberPrecision(7)  << asirasData.cBaselineLongitude.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.cBaselineAltitude.at(echo)
                << qSetFieldWidth(11) << qSetRealNumberPrecision(6)  << asirasData.altitudeRate.at(echo) //0
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.xVelocityITRF.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.yVelocityITRF.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.zVelocityITRF.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.xRealAntBeamDir.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.yRealAntBeamDir.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.zRealAntBeamDir.at(echo)
                << qSetFieldWidth(12) << qSetRealNumberPrecision(6)  << asirasData.xInterferometerBaseline.at(echo)
                << qSetFieldWidth(12) << qSetRealNumberPrecision(6)  << asirasData.yInterferometerBaseline.at(echo)
                << qSetFieldWidth(12) << qSetRealNumberPrecision(6)  << asirasData.zInterferometerBaseline.at(echo)
                << qSetFieldWidth(12) << qSetRealNumberPrecision(0)  << asirasData.measurementConfident.at(echo)
                << qSetFieldWidth(15) << qSetRealNumberPrecision(12) << asirasData.windowDelay.at(echo)
                << qSetFieldWidth(8)  << qSetRealNumberPrecision(2)  << asirasData.oCOGWidth.at(echo)
                << qSetFieldWidth(11) << qSetRealNumberPrecision(3)  << asirasData.retrackerRange.at(echo)
                << qSetFieldWidth(11) << qSetRealNumberPrecision(3)  << asirasData.derivedSurfaceElevation.at(echo)
                << qSetFieldWidth(8)  << qSetRealNumberPrecision(2)  << asirasData.aGCChannel1.at(echo)
                << qSetFieldWidth(8)  << qSetRealNumberPrecision(2)  << asirasData.aGCChannel2.at(echo)
                << qSetFieldWidth(8)  << qSetRealNumberPrecision(2)  << asirasData.totalFixedGain1.at(echo)
                << qSetFieldWidth(8)  << qSetRealNumberPrecision(2)  << asirasData.totalFixedGain2.at(echo)
                << qSetFieldWidth(8)  << qSetRealNumberPrecision(2)  << asirasData.transmitPower.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.dopplerRangeCorrection.at(echo) //0
                << qSetFieldWidth(7)  << qSetRealNumberPrecision(3)  << asirasData.instrumentRangeCorrectionCh1.at(echo)
                << qSetFieldWidth(7)  << qSetRealNumberPrecision(3)  << asirasData.instrumentRangeCorrectionCh2.at(echo)
                << qSetFieldWidth(10) << qSetRealNumberPrecision(6)  << asirasData.internalPhaseCorrection.at(echo) //0
                << qSetFieldWidth(10) << qSetRealNumberPrecision(6)  << asirasData.externalPhaseCorrection.at(echo) //0
                << qSetFieldWidth(14) << qSetRealNumberPrecision(2)  << asirasData.noisePower.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.roll.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.pitch.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.yaw.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.heading.at(echo)
                << qSetFieldWidth(7)  << qSetRealNumberPrecision(4)  << asirasData.stdDevRoll.at(echo)
                << qSetFieldWidth(7)  << qSetRealNumberPrecision(4)  << asirasData.stdDevPitch.at(echo)
                << qSetFieldWidth(7)  << qSetRealNumberPrecision(4)  << asirasData.stdDevYaw.at(echo)
                << qSetFieldWidth(12) << qSetRealNumberPrecision(9)  << asirasData.scaleFactor(echo)
                << qSetFieldWidth(4)  << qSetRealNumberPrecision(0)  << asirasData.numberOfMultilookedEchoes.at(echo)
                << qSetFieldWidth(12) << qSetRealNumberPrecision(0)  << asirasData.flags.at(echo)
                << qSetFieldWidth(8)  << qSetRealNumberPrecision(2)  << asirasData.beamBehaviourParameters.at(echo).at(0)/100.0
                << qSetFieldWidth(8)  << qSetRealNumberPrecision(2)  << asirasData.beamBehaviourParameters.at(echo).at(1)/100.0
                << qSetFieldWidth(8)  << qSetRealNumberPrecision(2)  << asirasData.beamBehaviourParameters.at(echo).at(2)
                << qSetFieldWidth(8)  << qSetRealNumberPrecision(2)  << asirasData.beamBehaviourParameters.at(echo).at(3)/100.0
                << qSetFieldWidth(8)  << qSetRealNumberPrecision(2)  << asirasData.beamBehaviourParameters.at(echo).at(4)/100.0;
        if(useGeoide)
            outputStream <<  fixed
                    << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << geoide.interpolateValue(asirasData.cBaselineLongitude.at(echo), asirasData.cBaselineLatitude.at(echo));
        outputStream<< endl;
    }
    else for(int echo=0; echo<asirasData.echoes(); echo++)
    {
        outputStream <<  fixed
                << qSetFieldWidth(10) << qSetRealNumberPrecision(3)  << asirasData.seconds.at(echo)
                << qSetFieldWidth(6)  << qSetRealNumberPrecision(0)  << asirasData.burstCounter.at(echo)
                << qSetFieldWidth(13) << qSetRealNumberPrecision(7)  << asirasData.cBaselineLatitude.at(echo)
                << qSetFieldWidth(13) << qSetRealNumberPrecision(7)  << asirasData.cBaselineLongitude.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.cBaselineAltitude.at(echo)
                << qSetFieldWidth(8)  << qSetRealNumberPrecision(2)  << asirasData.oCOGWidth.at(echo)
                << qSetFieldWidth(12) << qSetRealNumberPrecision(9)  << asirasData.scaleFactor(echo)
                << qSetFieldWidth(11) << qSetRealNumberPrecision(3)  << asirasData.retrackerRange.at(echo)
                << qSetFieldWidth(11) << qSetRealNumberPrecision(3)  << asirasData.derivedSurfaceElevation.at(echo)
                << qSetFieldWidth(14) << qSetRealNumberPrecision(2)  << asirasData.noisePower.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.roll.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.pitch.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.yaw.at(echo)
                << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << asirasData.heading.at(echo)
                << qSetFieldWidth(7)  << qSetRealNumberPrecision(4)  << asirasData.stdDevRoll.at(echo)
                << qSetFieldWidth(7)  << qSetRealNumberPrecision(4)  << asirasData.stdDevPitch.at(echo)
                << qSetFieldWidth(7)  << qSetRealNumberPrecision(4)  << asirasData.stdDevYaw.at(echo)
                << qSetFieldWidth(12) << qSetRealNumberPrecision(9)  << asirasData.scaleFactor(echo)
                << qSetFieldWidth(4)  << qSetRealNumberPrecision(0)  << asirasData.numberOfMultilookedEchoes.at(echo);
        if(useGeoide)
            outputStream <<  fixed
                    << qSetFieldWidth(9)  << qSetRealNumberPrecision(3)  << geoide.interpolateValue(asirasData.cBaselineLongitude.at(echo), asirasData.cBaselineLatitude.at(echo));
        outputStream<< endl;
    }


    outputFile.close();

    return(0);
}
